# New methods for News::Article
# $Id: article.pm,v 1.6 2000/10/15 19:53:37 md Exp $

package News::Article;
use strict;

sub sign_body {
	my ($self, %args) = @_;

	require GnuPG::Interface;
	require IO::Handle;
	my $gnupg = GnuPG::Interface->new;
	my ($input, $output, $error) =
		(IO::Handle->new, IO::Handle->new, IO::Handle->new);
	my $handles = GnuPG::Handles->new(stdin => $input, stdout => $output,
		stderr => $error);

	local $SIG{PIPE} = 'IGNORE';
	$gnupg->options->meta_interactive(0);
	$gnupg->options->hash_init(homedir => $args{Homedir}) if $args{Homedir};
	$gnupg->options->default_key($args{Key}) if $args{Key};
	$gnupg->passphrase($args{Pass});
	$gnupg->clearsign(handles => $handles);
	print $input "$_\n" foreach (@{$self->body});
	close $input;
	my @newbody = <$output>; close $output;
	my @err = <$error>; close $error;
	if (wait < 0 or $? > 0) {
		${$args{Errors}} = join('', @err) if exists $args{Errors};
		return '';
	}
	$self->set_body(@newbody);
	return 1;
}

sub verify_body {
	my ($self, %args) = @_;
	
	$self->{signer} = undef;
	require GnuPG::Interface;
	require IO::Handle;
	my $gnupg = GnuPG::Interface->new;
	my ($input, $output, $error, $status) =
		(IO::Handle->new, IO::Handle->new, IO::Handle->new, IO::Handle->new);
	my $handles = GnuPG::Handles->new(stdin => $input, stdout => $output,
		stderr => $error, status => $status);

	local $SIG{PIPE} = 'IGNORE';
	$gnupg->options->meta_interactive(0);
	$gnupg->options->hash_init(homedir => $args{Homedir}) if $args{Homedir};
	$gnupg->decrypt(handles => $handles);
	print $input "$_\n" foreach (@{$self->body});
	close $input;
	my @newbody = <$output>; close $output;
	my @err = <$error>; close $error;
	my @st = <$status>; close $status;
	%{$args{KeyInfo}} = KeyInfoParser(\@st) if exists $args{KeyInfo};
	if (wait < 0 or $? > 0) {
		${$args{Errors}} = join('', @err) if exists $args{Errors};
		return '';
	}
	$self->set_body(@newbody) if $args{RemoveSig};
	return 1;
}

sub KeyInfoParser {
	my ($st) = @_;
	my %ki;
	foreach (@$st) {
		chomp; s/^\[GNUPG:\]\s+//;
		if (/^(?:GOOD|BAD|ERR)SIG/) {
			my ($a, @b) = split(/\s+/, $_, 2);
			$ki{$a} = \@b;
		} elsif (/^VALIDSIG/ || /^SIG_ID/) {
			my ($a, @b) = split(/\s+/, $_, 3);
			$ki{$a} = \@b;
		} elsif (/^TRUST_(\w+)$/) {
			$ki{TRUST} = $1;
		} else {
			my ($a, $b) = split(/\s+/, $_, 2);
			$a ||= $_;
			$ki{$a} = $b;
		}
	}
	return %ki;
}

sub arttext {
	my $self = shift;
	return join("\n", $self->headers, '', @{$self->body});
}

sub writefile {
	my ($self, $file) = @_;
	open(OUT, ">$file") or return undef;
	$self->write(\*OUT);
	close OUT or return undef;
	1;
}

sub mail {
	my ($self, @recipients) = @_;

	if ($ENV{GCNDEBUG}) {
		print STDERR "------------->mail:\n"; $self->write(\*STDERR); return 1;
	}
	if (not @recipients) {
		foreach ($self->header('to'), $self->header('cc')) {
			my $addr = parseaddr($_);
			push @recipients, $addr if $addr;
		}
		if (not @recipients) {
			$self->{Err} = 'Cannot determine recipients'; return undef;
		}
	}

	require Net::SMTP;
	if (not $::SMTPConn) {
		$::SMTPConn = Net::SMTP->new(
			$::SMTPServer, Hello => $::hostname
# XXX		$self->{SMTPServer}, Hello => $self->{Hostname}
		);
		unless ($::SMTPConn) {
			$self->{Err} = 'Cannot connect to mail server'; return undef;
		}
	}
	unless ($::SMTPConn->mail($self->{Envelope}) and
			$::SMTPConn->to(@recipients) and
			$::SMTPConn->data(map {"$_\n"} ($self->headers, '', $self->body))){
		$self->{Err} = $::SMTPConn->message;
		$::SMTPConn->quit;
		return undef;
	}
	1;
}

# returns the last error message
sub error { shift->{Err} }

sub parseaddr {
	local $_ = shift;
	my $hdr = (/<(\S+)>/)[0];			# XXX should use Mail::Address
	$hdr = (split(' '))[0] unless $hdr;
	$hdr;
}

# workaround for a bug exposed by perl 5.6
sub News::FormReply::DESTROY { }

1;
