/*
 * artspooler.c - Copyright 1998 by Marco d'Itri <md@linux.it>
 * You can use this software under the terms of the GPL. If we meet some day,
 * and you think this stuff is worth it, you can buy me a beer in return.
 * $Id: artspooler.c,v 1.5 2000/04/09 16:41:55 md Exp $
 *
 * Esce sempre con EX_TEMPFAIL per non perdere MAI posta.
 * Si presume che il PID non si ripeta nell'arco di un secondo.
 * Deve essere suid e sgid all'owner della directory.
 * Periodicamente bisogna cancellare da directory/tmp/ i file piu` vecchi
 *  di un giorno.
 */

/*
 * Sotto linux dovrebbe essere sempre definito, ma i kernel 2.0.x lo
 * implementano come un fsync dell'intero disco con ovvie conseguenze
 * sulle prestazioni del sistema.
 * I file system derivati da FFS di BSD aggiornano sempre i metadati in
 * modo sincrono.
 */

/* #define FSYNCDIR */

#include <sys/stat.h>		/* mkdir */
#include <stdio.h>		/* fputs, printf, snprintf, perror */
#include <unistd.h>		/* getpid, fdatasync, close, write, unlink */
#include <fcntl.h>		/* open */
#include <time.h>		/* time */
#include <errno.h>		/* EIO */
#include <sysexits.h>		/* EX_TEMPFAIL 75 */

/* work around a missing fdatasync(2) */
#ifndef _POSIX_SYNCHRONIZED_IO
#  define fdatasync	fsync
#endif

void progerr(const char *);
void syserr(const char *, const char *);

void progerr(const char *msg) {
    fputs(msg, stderr);
    exit(EX_TEMPFAIL);
}

void syserr(const char *msg, const char *file) {
    perror(msg);
    if (unlink(file) < 0)
	perror("unlink");
    exit(EX_TEMPFAIL);
}

int main(int argc, char *argv[])
{
    int fd;
    unsigned int n;
    char buf[BUFSIZ];
    char file[FILENAME_MAX], file2[FILENAME_MAX];
    pid_t mypid = getpid();
    time_t mytime = time(NULL);

    if (argc != 3)
	progerr("Uso: artspooler directory it.nome.gruppo\n");
    if ((snprintf(file, FILENAME_MAX, "%s/tmp", argv[1])) == -1)
	progerr("Nome della directory troppo lungo!\n");
    mkdir(file, 0700);		/* lazy directory creation */
    if ((snprintf(file, FILENAME_MAX, "%s/tmp/%ld.%d-%s", argv[1],
		    (long) mytime, (int) mypid, argv[2])) == -1)
	progerr("Nome del file troppo lungo!\n");
    if ((fd = open(file, O_CREAT | O_EXCL | O_WRONLY, 0600)) < 0)
	syserr("open", file);
    while ((n = read(0, buf, BUFSIZ)) > 0)
	if (write(fd, buf, n) != n) {
	    close(fd);
	    syserr("write", file);
	}
    if (n < 0)
	syserr("read", file);
    if ((fdatasync(fd)) < 0)
	syserr("fdatasync", file);
    if ((close(fd)) < 0)
	syserr("close", file);
    if ((snprintf(file2, FILENAME_MAX, "%s/%ld.%d-%s", argv[1],
		    (long) mytime, (int) mypid, argv[2])) == -1) {
	if (unlink(file) < 0)
	    perror("unlink");
	progerr("Si e` ristretto il buffer?!?\n");
    }
    if ((rename(file, file2)) < 0)
	syserr("rename", file);
#ifdef FSYNCDIR
    if ((fd = open(argv[1], O_RDONLY, 0666)) < 0)
	syserr("open dir", file2);
    if ((fsync(fd)) < 0 && errno == EIO)
	syserr("fsync dir", file2);
    close(fd);
#endif
    exit(0);
}
