# $Id: common.pm,v 1.19 2000/10/15 19:54:20 md Exp $

package main;
use strict;
use vars qw($GCNPath);

$GCNPath = '/opt/gcn';

##############################################################################
# Legge il database con i percorsi dei gruppi.
sub readgroupdb {
}

sub groupdatadir ($) {
	my $g = shift;

	return undef unless $g =~ /^([-_+\.\/a-zA-Z0-9]+)$/; # belt AND suspenders
	return $GCNPath."/groups/$g" || undef;
}

##############################################################################
# readgroupdata(\%dati, $nomegruppo);
# Mette in %dati i dati del gruppo $nomegruppo, prendendoli dal database.

sub readgroupdata (\%$) {
	my ($dati, $nome) = @_;
	return "$nome does not exist"
		unless -f "$GCNPath/schede/$nome";
	open(GROUPDATAFILE, "$GCNPath/schede/$nome")
		or return "can't open data for $nome: $!";
	my $r = parsegroupdesc($dati, \*GROUPDATAFILE);
	close GROUPDATAFILE;
	return $r;
}

##############################################################################
sub dumpdir {
	my $dir = shift;
	my @l;

	chdir("$dir/spool") or return undef;
	opendir(DIR, '.') or return undef;
	while (my $f = readdir(DIR)) {
		next if ($f eq '.' or $f eq '..' or $f eq 'tmp');
		my @sb = stat($f);
		next unless @sb;
		open(FILE, $f) or next;
		my ($a, $s);
		while (<FILE>) {
			$a = $1 if /^From: (.*)/;
			$s = $1 if /^Subject: (.*)/;
			last if (($a and $s) or /^$/);
		}
		close FILE;
		next unless ($a and $s);
		push @l, [ $f, $sb[7], $sb[9], $a, $s ];
	}
	closedir DIR;
	return \@l;
}

##############################################################################
# parsegroupdesc(\%dati, $file);
# Legge dal $file i dati separati da %TAG e li salva nell'array %dati

sub parsegroupdesc (\%$) {
	my ($dati, $file) = @_;
	my $type;
	my %convtable = (
		n => 'name', t => 'title', r => 'retry', m => 'mods',
		p => 'proponents', g => 'rfdgroups', c => 'charter', 'o' => 'oldname'
	);

	while (<$file>) {
		my $keyword = 0;
		s/\s+\r?\n?$//;					# trim trailing blanks and chomp
		if (/^%(\w+)$/) {				# look for a keyword
			$type = lc $1;
			$type = $convtable{$type} if exists $convtable{$type};
			$keyword = 1;
		}

		if (not defined $type) {		# skip heading blank lines
			return "line out of context: $_" unless /^$/;
			next;
		}
		if (grep(/^$type$/, qw(autopilot norcheck pgpverify stripbody
				noreceipts mustregister usegoodsubjlist
				keepapproved keeprejected keepmodrejected))) {
			$dati->{$type} = 1;			# boolean
			undef $type; next;
		}
		next if (/^$/ and not $type =~ /(?:charter|note|footer)/);
		if (grep(/^$type$/, qw(name title retry followupto pgpmoose
				oldname roboaddr officialaddr maxsize))) {
			next if $keyword;
			$dati->{$type} = $_;		# scalar
			undef $type; next;
		}
		if (grep(/^$type$/, qw(mods proponents rfdgroups charter note
				footer keywords badformat))) {
			next if $keyword;
			push @{$dati->{$type}}, $_;	# list
			next;
		}
		return "unknown keyword: $type";
	}

	if (exists $dati->{mods} and $dati->{mods}[0] eq 'P') {
		return 'No proponents to promote as moderators have been defined'
			if not exists $dati->{proponents};
		shift @{$dati->{mods}}; # remove the fake element
		push @{$dati->{mods}}, @{$dati->{proponents}};
	}
	# trim trailing blank lines
	foreach (qw(charter note footer)) {
		next if not exists $dati->{$_};
		pop @{$dati->{$_}} while (@{$dati->{$_}}->[$#{$dati->{$_}}] eq '');
	}
	return '';
}

sub checkgroupdata ($$) {
	my ($dati, $willcheck) = @_;
	my $tocheck = $willcheck || [ qw(proponents charter name title) ];

	foreach (@$tocheck) {
		return "field $_ is mandatory" if not $dati->{$_};
	}
	if ($dati->{name}) {
		return 'bad characters in the name field'
			if $dati->{name} !~ /^(?:it|linux)\.[-\.a-z0-9]+$/;
		return 'two consecutive dots in the name field'
			if $dati->{name} =~ /\.\./;
	}
	foreach (split(/\./, $dati->{name})) {
		return "element $_ is too long\n" if length($_) > 20;
	}
	my $len = length($dati->{name}) + 8 - length($dati->{name}) % 8;
	$len = 24 if $len < 24;
	$len += length $dati->{'title'};
	return "title too long ($len)\n" if $len > 79;
	return 'title field must not have a trailing dot'
		if $dati->{'title'} =~ /[^\.]\.$/;
	return 'title field must not have trailing "(Moderated)" string'
		if $dati->{'title'} =~ /\(Moderated\)$/;
	return 'retry field must be a number'
		if ($dati->{retry} and $dati->{retry} !~ /^\d+$/);
	return '';
}

##############################################################################
# Stampa il messaggio di errore @msg ed esce con status 75.

sub tempfail {
	$SIG{__DIE__} = 'DEFAULT';	# evita un loop
	require Carp;
	Carp::cluck("@_\n\n\n");
	exit 75;
}
1;
