# New methods for News::Gateway
# $Id: gateway.pm,v 1.18 2000/10/15 19:54:35 md Exp $

package News::Gateway;
use strict;

##############################################################################
$HOOKS{ctlhdr} = [];
sub ctlhdr_mesg {
	my $self = shift;
	my $article = $self->{article};

	my $h = $article->header('X-Robomod') || return undef;
	$article->drop_headers('X-Robomod');
	$self->{CtlHdr} = { split_header_words($h) };
	undef;
}

sub mail_filereply {
	my ($self, $filename) = @_;
	require News::FormReply;
	my $article = $self->{article};
	my $source = { SUBJECT => $article->header ('subject') };
	my $reply = News::FormReply->new($article, $filename, $source, @_)
		or $self->error("Unable to generate reply from $filename");
	$reply->envelope($$self{envelope}) if $$self{envelope};
	$reply->set_headers(precedence => 'junk');
	$reply->set_headers(to => $self->{CtlHdr}{replyto})
		if $self->{CtlHdr}{replyto};
	$reply->mail();
}

##############################################################################
# checks for crossposts in moderated groups

$HOOKS{modxpost} = ['moderatedlist'];
sub modxpost_conf {
	my ($self, $directive, $a) = @_;
	no strict 'refs';	# XXX
	$self->{Moderatedlist} = \%{$a};
	use strict;
	undef;
}

sub modxpost_mesg {
	my $self = shift;
	my $article = $self->{article};

	my $ng = $article->header('newsgroups') || return undef;
	my $skip = $self->primarygroup || return undef;

	foreach (split(/,/, $ng)) {
		next if $_ eq $skip;
		return "Crossposted to another moderated group $_"
			if exists $self->{Moderatedlist}->{$_};
	}
	undef;
}

##############################################################################
# PGP related modules

=head1
$HOOKS{pgpverify} = ['pgpverify'];
sub pgpverify_conf {
	my ($self, $directive, $a, $b) = @_;
	if ($a eq 'mustsign') {
		$self->{pgpverify}{mustsign} = 1;
	} else {
		open(PGPVERIFY, $a)
			or $self->error("Can't open authorized keys file $a: $!");
		local $_;
		while (<PGPVERIFY>) {
			chomp;
			push @{$self{pgpverify}{list}}, $_;
		}
		close PGPVERIFY;
	}
}

sub pgpverify_mesg {
	my $self = shift;
	my $article = $self->{article};

	my $e = $article->verify_body(Homedir => $GCNPath.'/groups/gnupg/');
	return "No signature found (or system error?)\n$e" if ($e);
	my $signer = $article->signer;
	foreach (@{$self{pgpverify}{list}}) {
		return undef if ($signer =~ /$_/i);
	}
	return 'User not authorized: '.$signer;
}

##############################################################################
# misc things

$HOOKS{fido} = [];
sub fido_mesg {
	my $self = shift;
	my $article = $self->{article};

	my $ng = $article->header('newsgroups');
	return 'rong' if ( $ng =~ /^fido\.coord/
					or $ng eq 'fido.ita.news'
					or $ng eq 'fido.ita.tech_net'
					or $ng eq 'fido.ita.ping_033');
}
=cut

$HOOKS{gcn} = [];
sub gcn_mesg {
	my $self = shift;
	my $article = $self->{article};

	# "(none)" is generated by the mailtonews module
	local $_ = $article->header('subject');
	return 'nosubject' if /^\(none\)$/;
	return 'subjectre' if /^R: / and not $::GroupData{norcheck};
	s/Re?:\s+//;	# remove R:/Re: prefix
	s/\[.*\]//;		# remove tags
	return 'badsubject' if /(?:[\$\?\!]){3,}/ or (tr/A-Z// > tr/A-Z//c + 8);
	return 'stile' if /^\[OT\]/;

	$_ = lc $article->header('content-type');
	return 'html' if $_ and (/multipart\/alternative/ or /html/);
	return 'multipart' if /multipart\/mixed/;

	my $body = $article->body;

	my $q = 0;
	foreach (@$body) {
		next if /^\s*[\>:%]?\s*$/;		# skip blank lines
		if ((/^\s*[:%\>]/) or (/^\s*\w{0,9}>/)) { $q++; next; }
		if ($q) { $q = 0; last; }		# found text after quoted text
		# no quoted text found yet ($q is 0)
	}
	return 'stile' if $q > 4;

	if (scalar @$body > 20) { # check quoting ratio and article size
		my $qlnum = 0; my $artsize = 0;
		my $name;					# check only the first encoded name
		foreach (@$body) {
			$qlnum++ if /^\s*[:%\>]/ or /^\s*\w{0,9}>/;
			$artsize += length;
			if (not defined $name and /^begin\s+\d\d\d\s+(.*)/) {
				$name = $1;
				return 'badformat' if $name =~ / /;
				foreach (@{$::GroupData{badformat}}) {
					return 'badformat' if $name =~ /$_$/;
				}
			}
		}
		return 'quoting' if ($qlnum/(scalar @$body + 10) > 0.6);
		return 'size' if $artsize > ($::GroupData{maxsize} || 1_000_000);
	}

	$article->rename_header('Date', 'X-Original-Date', 'clobber')
		if $article->header('newsgroups') =~ /^it\.binari\.x\.erotismo/;
	my $org = $article->header('organization');
	$article->set_headers('organization' => 
		"Robomoderatore (by Md)") if not $org or
		$org eq	'Centro di Servizi per la rete di Ateneo - Pisa - Italy';

	undef;
}

$HOOKS{footer} = [];
sub footer_mesg {
	return undef unless $::GroupData{footer};
	my $body = shift->{article}->body;
	push @$body, @{$::GroupData{footer}};
	undef;
}

##############################################################################
# new variables

sub moderator {
	my $self = shift;
	$$self{Moderator} = shift if @_;
	$$self{Moderator};
}

sub primarygroup {
	my $self = shift;
	$$self{Primarygroup} = shift if @_;
	$$self{Primarygroup};
}

##############################################################################
# reads keywords from the template
sub keywords_conf2 {
	my ($self, $keywords) = @_;
	foreach my $k (@$keywords) {
		$$self{keywords}{valid}{lc $k} = 1;
	}
}

# process a list of configuration lines
sub config_lines {
	my ($self, $data) = @_;
	foreach (@$data) {
		s/^\s+//;
		next if /^\#/ or /^$/;
		s/\s+$//;
		$self->config_line($_);
	}
}

# post an article, with connection caching
sub cpost {
	my ($self, $server) = @_;

	if ($ENV{GCNDEBUG}) {
		print STDERR "------------->post:\n"; $self->write(\*STDERR); return 1;
	}
	$::NNTPConn = $server if defined $server;
	if (not defined $::NNTPConn) {
		require Net::NNTP;
		$::NNTPConn = Net::NNTP->new;
		return 'Unable to connect to server' unless $::NNTPConn;
	}
	return $self->post($::NNTPConn);
}

##############################################################################
# select a moderator
sub assign_mod {
	my ($self, $filename, $mlist) = @_;

	# if there is only one moderator, we need no database.
	if (not $filename or scalar @$mlist eq 1 and 0) {
		$$self{Moderator} = $mlist->[int rand scalar @$mlist];
		return 1;
	}
	
	# tie the database, if this has not been done yet
	unless ($::ID_DB) {
		require DB_File;
		tie %::ID_DB, 'DB_File', $filename,
				Fcntl::O_CREAT()|Fcntl::O_RDWR(), 0644, $DB_File::DB_HASH
			or return undef;
	}

	# try all Message-IDs in the References header, starting from the last one
	my $art = $self->{article};
	if (my $references = $art->header('references')) {
		foreach (reverse split(/[\s\n\r]+/, $references)) {
			if (my $v = $::ID_DB{$_}) {
				my $mod = (split(/<>/, $v))[0];
				if (grep(/^\Q$mod\E$/, @$mlist)) {
					$$self{Moderator} = $mod; return 1;
				}
			}
		}
	}
	# select a random moderator
	$$self{Moderator} = $mlist->[int rand scalar @$mlist];
	$::ID_DB{$art->header('message-id')} = $$self{Moderator} . '<>' . time;
	return 1;
}

##############################################################################
# *list modules for News::Gateway

$HOOKS{whitelist} = ['whitelist'];
sub whitelist_conf {
	my ($self, $directive, $path) = @_;
	$self->list_parse('white', $path);
}

sub whitelist_mesg {
	shift->list_search('white');
}

$HOOKS{blacklist} = ['blacklist'];
sub blacklist_conf {
	my ($self, $directive, $path) = @_;
	$self->list_parse('black', $path);
}

sub blacklist_mesg {
	shift->list_search('black', 1);
}

$HOOKS{nukelist} = ['nukelist'];
sub nukelist_conf {
	my ($self, $directive, $path) = @_;
	$self->list_parse('nuke', $path);
}

sub nukelist_mesg {
	shift->list_search('nuke', 1);
}

$HOOKS{goodsubjlist} = ['badsubjlist'];
sub goodsubjlist_conf {
	my ($self, $directive, $path) = @_;
	$self->list_parse('goodsubj', $path);
}

sub goodsubjlist_mesg {
	shift->list_search('goodsubj', 0, 'subject');
}

$HOOKS{badsubjlist} = ['badsubjlist'];
sub badsubjlist_conf {
	my ($self, $directive, $path) = @_;
	$self->list_parse('badsubj', $path);
}

sub badsubjlist_mesg {
	shift->list_search('badsubj', 1, 'subject');
}

sub list_search {
	my ($self, $list, $type, $header, $msg) = @_;
	my $hdr;

	$header ||= 'from';
	if ($header eq 'from') {
		$hdr = News::Article::parseaddr($self->{article}->header('from'));
	} else {
		$hdr = $self->{article}->header($header);
	}

	if ($type) {
		foreach (@{$self->{"${list}list"}}) {
			return $msg ? "$msg: $hdr" : "Rejected: $hdr" if ($hdr =~ /$_/i);
		}
		return undef;
	} else {
		foreach (@{$self->{"${list}list"}}) {	# untrusted regexps may cause
			return undef if $hdr =~ /$_/i;		#  run time errors!
		}
		return $msg ? "$msg: $hdr" : "Unknown: $hdr";
	}
}

sub list_parse {
	my ($self, $list, $file) = @_;

	open(LIST, $file) or $self->error("Can't open ${list}list file $file: $!");
	local $_;
	while (<LIST>) {
		chomp;
		next if /^$/;
		push @{$self->{"${list}list"}}, $_;
	}
	close LIST;
}

##############################################################################
# originally from HTTP::Headers::Util
# Copyright 1997-1998, Gisle Aas
sub split_header_words {
	local $_ = shift;
	my @cur;

	while (length) {
		if (s/^\s*(=*[^\s=;,]+)//) {	# 'token' or parameter 'attribute'
			push @cur, $1;		# key
			my $val;
			if (s/^\s*=\s*\"([^\"\\]*(?:\\.[^\"\\]*)*)\"//) {
				$val = $1; $val =~ s/\\(.)/$1/g;	# quoted value
			} elsif (s/^\s*=\s*([^;,\s]*)//) {
				$val = $1; $val =~ s/\s+$//;	# unquoted value
			} else {							#no value, a lone token
				$val = undef;
			}
			push @cur, $val;	# value
		} elsif (s/^\s*[,;]// || s/^\s+//) {
			# continue
		} else {
			#die "This should not happen: '$_'";
		}
	}
	@cur;
}

1;
